import gleam/dynamic/decode.{type Decoder}
import gleam/json.{type Json}
import gleam/time/calendar
import gleam/time/timestamp

pub type Timestamp =
  timestamp.Timestamp

pub fn parse(value: String) -> Result(Timestamp, Nil) {
  timestamp.parse_rfc3339(value)
}

pub fn from_unix(seconds: Int) -> Timestamp {
  timestamp.from_unix_seconds(seconds)
}

pub fn decoder() -> Decoder(Timestamp) {
  use value <- decode.then(decode.string)
  case parse(value) {
    Ok(timestamp) -> decode.success(timestamp)
    Error(Nil) -> decode.failure(timestamp.from_unix_seconds(0), "Timestamp")
  }
}

pub fn encode(timestamp: Timestamp) -> Json {
  json.string(timestamp.to_rfc3339(timestamp, calendar.utc_offset))
}
